%--------------------------------------------------------------------
% PURPOSE: This is the main file for testing the FRD validity assumptions
% using Arai, Hsu, Kitagawa, Mourifie, and Wan (2020)
% Here we calculate the jump size of the propensity score.
%--------------------------------------------------------------------


clc
clear
rng(12345)


%##########################################################################
rich=15;                    % Parameter Q in the interval class
nbs=1000;                   %BOOTSTRAP REPLICATIONS
vweight=1;                   % vweight=1 is using variance weighting. vweight=0 means no weighting.
smooth=4.5;                       % undersmoothing constant.
band={'three','five','AI','IK','CCT'}; % Five bandwidth choices reported in the main text
% band={'AI-CER','IK-CER','CCT-CER','AI-RBC','IK-RBC','CCT-RBC'}; % other possible choice of bandwidths.
CAT={'g4_math','g5_math'}; % Four subjects: 4_math, g4_verb, g5_math or g5_verb
CUTOFF=[40, 80, 120]; % Three cutoff values
if vweight==0
    XI=1;
else
    %     XI=[0.00999950,0.03160696,0.17058722,0.5];
    XI=0.00999950;
end
VW={'no weighting','weigthing'};
nband=length(band);         % # of bandwidths
numcat=2;                   % # of catogories: g.
numcutoff=3;                 % # of cutoff values: 40, 80, 120
numxi=length(XI);             % # of xi parameters to use
pwd;
load('angrist_levy_data.mat')

disp(['bandwidth type',band])
%#########################################################################
for cat=1:numcat
    file=CAT{cat};
    Data=eval(file);
    Y=Data(:,1); % outcome variable
    sdy=(Y-mean(Y))./std(Y);
    S=Data(:,3);
    R=Data(:,4); % Running variable
    D=Data(:,2); % Treatment variable
    for cutoff=1:numcutoff
        c=CUTOFF(cutoff)+1;
        I=find(D==cutoff|D==cutoff+1);
        y=sdy(I);
        t=D(I)-cutoff;
        x=R(I);
        s=S(I);
        
        JS=nan(1,nband);
        n=length(y);
        n_plus=sum((x>=c));
        n_minus=sum((x<c));
        B=zeros(2,4);
        for nb=1:nband
            bandwidth=band(nb);
            if strcmp(bandwidth,'three')==1
                bw=[3;3];
            elseif strcmp(bandwidth,'five')==1
                bw=[5;5];
            elseif strcmp(bandwidth,'AI')==1
                output = rddmmsef(x,y,t,c);
                bws=[output.h(2),output.h(1)];
                bw=[min(bws(1)*n_minus^(1/5-1/smooth),15),min(bws(2)*n_plus^(1/5-1/smooth),15)];
                % Here AI bandwidth is regularized by no larger than 15
            elseif strcmp(bandwidth,'IK')==1
                bws = rd_optbandwidth_1(y,x,c);
                bw=[bws*n_minus^(1/5-1/smooth),bws*n_plus^(1/5-1/smooth)];
            elseif strcmp(bandwidth,'CCT')==1
                output = rddcct(x,y,c);
                bws=output.h;
                bw=[bws*n_minus^(1/5-1/smooth),bws*n_plus^(1/5-1/smooth)];
            elseif strcmp(bandwidth,'AI-CER')==1
                output = rddmmsef(x,y,t,c);
                bws=[output.h(2),output.h(1)];
                bw=[bws(1)*n_minus^(-1/20),bws(2)*n_plus^(-1/20)];
            elseif strcmp(bandwidth,'IK-CER')==1
                bws = rd_optbandwidth_1(y,x,c);
                bw=[bws*n_minus^(-1/20),bws*n_plus^(-1/20)];
            elseif strcmp(bandwidth,'CCT-CER')==1
                output = rddcct(x,y,c);
                bws=output.h;
                bw=[bws*n_minus^(-1/20),bws*n_plus^(-1/20)];
            elseif strcmp(bandwidth,'AI-RBC')==1
                output = rddmmsef(x,y,t,c);
                bws=[output.h(2),output.h(1)];
                bw=bws;
            elseif strcmp(bandwidth,'IK-RBC')==1
                bws = rd_optbandwidth_1(y,x,c);
                bw=[bws,bws];
            elseif strcmp(bandwidth,'CCT-RBC')==1
                output = rddcct(x,y,c);
                bws=output.h;
                bw=[bws,bws];
            end
            
            
           
            % Here we calculate the difference of right- and left-average
            % of the treatment variable, where the average calculated within in the
            % right- and left-bandwidth distance from the cutoff. 
            % For example if bandwdith=3 and cutoff is 40, then right
            % average is calculated among schools with 40, 41, and 42
            % enrolled students; and left-average is calculated among
            % schools with 39, 38, and 37 students. 
            I=find((x<=c+bw(2)).*(x>=c)==1);
            plus=mean(t(I));
            I=find((x<=c-1).*(x>c-bw(1)-1)==1);
            minus=mean(t(I));
            
            JS(nb) =plus-minus;
            
        end

        disp([file,'   ',num2str(40*cutoff),'  ',num2str(JS)])
        
    end
    
end

beep;
