function output = rddcct(X,Y,c)
    
% Imput
% X: the running variable
% Y: Output variable
% c: cut-off point

% Output
%    output.h: the main bandwidth
%    output.b: pilot bandwidths    
%    output.tau: rd estimate
%    output.delta: estimates on the right and left;

     n = size(X,1);
     Xbin_lt_x = (X<c);          % binary indicator for observations less than x
     Xbin_bt_x = (X>c);          % bigger than x

     Xx = X-c;
     Xx2 = Xx.^2;
     Xx3 = Xx.*Xx2;
     Xx4 = Xx2.^2;
     nones = ones(n,1);
     data_mat = [Y nones Xx Xx2 Xx3 Xx4];
     data_mat_p = data_mat(Xbin_bt_x==1,:); % data_mat only for positive X
     data_mat_m = data_mat(Xbin_lt_x==1,:); % data_mat only for negative X
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 0 Initial bandwidths
    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    Sx = sqrt(var(X));
    w0 = min(Sx,iqr(X)/1.349);
    vn = 2.58*w0*n^(-1/5); % OK
    
    % 4th-order global fit on each side
    e4 = [0 0 0 0 1]';    
    y0p = data_mat_p(:,1);
    X0p = data_mat_p(:,2:6);
    bptic = X0p\y0p;
    y0m = data_mat_m(:,1);
    X0m = data_mat_m(:,2:6);
    bmtic = X0m\y0m;
    
    % Get constant B33
    B330 = CCT_B(3,3)/6;
    
    % Compute inverse of Gamma(vn)
    e3 = [0 0 0 1]';
    v_vec = [1 vn^(-1) vn^(-2) vn^(-3)];
    X3v_p = data_mat_p(:,2:5)*diag(v_vec);
    X3v_m = data_mat_m(:,2:5)*diag(v_vec);
    Kvec_v_p = (1-abs(data_mat_p(:,3))/vn).*(abs(data_mat_p(:,3))<vn)/vn;
    Kvec_v_m = (1-abs(data_mat_m(:,3))/vn).*(abs(data_mat_m(:,3))<vn)/vn;
    Kindp = find(Kvec_v_p ~= 0);
    Kindm = find(Kvec_v_m ~= 0);
    
    y0p2 = y0p(Kindp,1);
    X0p2 = X0p(Kindp,:);
    X3v_p2 = X3v_p(Kindp,:);
    Kvec_v_p2 = Kvec_v_p(Kindp,1);
    y0m2 = y0m(Kindm,1);
    X0m2 = X0m(Kindm,:);
    X3v_m2 = X3v_m(Kindm,:);
    Kvec_v_m2 = Kvec_v_m(Kindm,1);
    Wv_p = diag(Kvec_v_p2);
    Wv_m = diag(Kvec_v_m2); 
    Gamma_p30 = X3v_p2'*Wv_p*X3v_p2/n;
    Gamma_m30 = X3v_m2'*Wv_m*X3v_m2/n;
    invGamma_p30 = invpd(Gamma_p30);
    invGamma_m30 = invpd(Gamma_m30);
    
    % Compute Psi(v)
    knn = 3;
    s2yyp_vec = (3/4)*(y0p2-nnm(y0p2,X0p2(:,2),knn)).^2;
    s2yym_vec = (3/4)*(y0m2-nnm(y0m2,X0m2(:,2),knn)).^2;
    Psi_yyp30 = X3v_p2'*diag((Kvec_v_p2.^2).*s2yyp_vec)*X3v_p2/n;
    Psi_yym30 = X3v_m2'*diag((Kvec_v_m2.^2).*s2yym_vec)*X3v_m2/n;
    
    % Compute V+,3,3 and V-,3,3
    Vp330 = (e3'*invGamma_p30*Psi_yyp30*invGamma_p30*e3)/n/vn^6;
    Vm330 = (e3'*invGamma_m30*Psi_yym30*invGamma_m30*e3)/n/vn^6;
    V330 = Vp330 + Vm330;
    
    % Compute Initial bandwidth
    C012 = 7*n*vn^7*V330/2/B330^2/(e4'*bptic-e4'*bmtic)^2;
    cn0 = C012^(1/9)*n^(-1/9);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 1 Pilot bandwidths b
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Get Constant B22
    B221 = CCT_B(2,2)/2;
    
    % local polynomial estimates
    Kvec_c_p = (1-abs(data_mat_p(:,3))/cn0).*(abs(data_mat_p(:,3))<cn0)/cn0;
    Kvec_c_m = (1-abs(data_mat_m(:,3))/cn0).*(abs(data_mat_m(:,3))<cn0)/cn0;
    X3c_p = data_mat_p(:,2:5);
    X3c_m = data_mat_m(:,2:5);
    Wvc_p = diag(Kvec_c_p);
    Wvc_m = diag(Kvec_c_m);
    betac_all_p = (X3c_p'*Wvc_p*X3c_p)\(X3c_p'*Wvc_p*y0p);
    betac_all_m = (X3c_m'*Wvc_m*X3c_m)\(X3c_m'*Wvc_m*y0m);
    
    % Compute V22(vn)
    X2v_p = X3v_p2(:,1:3);
    X2v_m = X3v_m2(:,1:3);
    Gamma_p21 = X2v_p'*Wv_p*X2v_p/n;
    Gamma_m21 = X2v_m'*Wv_m*X2v_m/n;
    invGamma_p21 = invpd(Gamma_p21);
    invGamma_m21 = invpd(Gamma_m21);
    e2 = [0 0 1]';
    Psi_yyp21 = X2v_p'*diag(Kvec_v_p2.^2.*s2yyp_vec)*X2v_p/n;
    Psi_yym21 = X2v_m'*diag(Kvec_v_m2.^2.*s2yym_vec)*X2v_m/n;
    Vp221 = (e2'*invGamma_p21*Psi_yyp21*invGamma_p21*e2)/n/vn^4;
    Vm221 = (e2'*invGamma_m21*Psi_yym21*invGamma_m21*e2)/n/vn^4;
    V221 = Vp221 + Vm221;
    
    % Compute V33(cn)
    c_vec = [1 cn0^(-1) cn0^(-2) cn0^(-3)];
    Kvec_c_p = (1-abs(data_mat_p(:,3))/cn0).*(abs(data_mat_p(:,3))<cn0)/cn0;
    Kvec_c_m = (1-abs(data_mat_m(:,3))/cn0).*(abs(data_mat_m(:,3))<cn0)/cn0;
    Kind_c_p = find(Kvec_c_p ~= 0);
    Kind_c_m = find(Kvec_c_m ~= 0);
    Wc_p = diag(Kvec_c_p(Kind_c_p,1));
    Wc_m = diag(Kvec_c_m(Kind_c_m,1));
    X3c_p = data_mat_p(Kind_c_p,2:5)*diag(c_vec);
    X3c_m = data_mat_m(Kind_c_m,2:5)*diag(c_vec);
    Gamma_p31 = X3c_p'*Wc_p*X3c_p/n;
    Gamma_m31 = X3c_m'*Wc_m*X3c_m/n;
    invGamma_p31 = invpd(Gamma_p31);
    invGamma_m31 = invpd(Gamma_m31);
    y0c_p = y0p(Kind_c_p,1);
    y0c_m = y0m(Kind_c_m,1);
    X0c_p = X0p(Kind_c_p,:);
    X0c_m = X0m(Kind_c_m,:);
    s2yypc_vec = (3/4)*(y0c_p-nnm(y0c_p,X0c_p(:,2),knn)).^2;
    s2yymc_vec = (3/4)*(y0c_m-nnm(y0c_m,X0c_m(:,2),knn)).^2;
    Kvec_c_p2 = Kvec_c_p(Kind_c_p,1);
    Kvec_c_m2 = Kvec_c_m(Kind_c_m,1);
    Psi_yyp31 = X3c_p'*diag(Kvec_c_p2.^2.*s2yypc_vec)*X3c_p/n;
    Psi_yym31 = X3c_m'*diag(Kvec_c_m2.^2.*s2yymc_vec)*X3c_m/n;
    Vp331 = 36*(e3'*invGamma_p31*Psi_yyp31*invGamma_p31*e3)/n/cn0^6;
    Vm331 = 36*(e3'*invGamma_m31*Psi_yym31*invGamma_m31*e3)/n/cn0^6;
    V331 = Vp331 + Vm331;

    C012 = 5*n*vn^5*V221/2/B221^2/((e3'*betac_all_p + e3'*betac_all_m)^2 + 3*V331/36);
    bcct = C012^(1/7)*n^(-1/7);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 2 Main bandwidths h
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Get Constant B01 
    B012 = CCT_B(0,1);
    b_vec = [1 bcct^(-1) bcct^(-2)];
    Kvec_b_p = (1-abs(data_mat_p(:,3))/bcct).*(abs(data_mat_p(:,3))<bcct)/bcct;
    Kvec_b_m = (1-abs(data_mat_m(:,3))/bcct).*(abs(data_mat_m(:,3))<bcct)/bcct;
    Kind_b_p = find(Kvec_b_p ~= 0);
    Kind_b_m = find(Kvec_b_m ~= 0);
    Wb_p = diag(Kvec_b_p(Kind_b_p,1));
    Wb_m = diag(Kvec_b_m(Kind_b_m,1));
    X2b_p = data_mat_p(Kind_b_p,2:4)*diag(b_vec);
    X2b_m = data_mat_m(Kind_b_m,2:4)*diag(b_vec);
    y0b_p = y0p(Kind_b_p,1);
    y0b_m = y0m(Kind_b_m,1);
    X0b_p = X0p(Kind_b_p,1:3);
    X0b_m = X0m(Kind_b_m,1:3);
    s2yypb_vec = (3/4)*(y0b_p-nnm(y0b_p,X0b_p(:,2),knn)).^2;
    s2yymb_vec = (3/4)*(y0b_m-nnm(y0b_m,X0b_m(:,2),knn)).^2;
    betab_all_p = (X0b_p'*Wb_p*X0b_p)\(X0b_p'*Wb_p*y0b_p);
    betab_all_m = (X0b_m'*Wb_m*X0b_m)\(X0b_m'*Wb_m*y0b_m);
    
    X1v_p = X3v_p2(:,1:2);
    X1v_m = X3v_m2(:,1:2);
    Gamma_p12 = X1v_p'*Wv_p*X1v_p/n;
    Gamma_m12 = X1v_m'*Wv_m*X1v_m/n;
    invGamma_p12 = invpd(Gamma_p12);
    invGamma_m12 = invpd(Gamma_m12);
    e0 = [1 0]';
    Psi_yyp12 = X1v_p'*diag(Kvec_v_p2.^2.*s2yyp_vec)*X1v_p/n;
    Psi_yym12 = X1v_m'*diag(Kvec_v_m2.^2.*s2yym_vec)*X1v_m/n;
    Vp012 = (e0'*invGamma_p12*Psi_yyp12*invGamma_p12*e0)/n;
    Vm012 = (e0'*invGamma_m12*Psi_yym12*invGamma_m12*e0)/n;
    V012 = Vp012 + Vm012;
    
    Gamma_p22 = X2b_p'*Wb_p*X2b_p/n;
    Gamma_m22 = X2b_m'*Wb_m*X2b_m/n;
    invGamma_p22 = invpd(Gamma_p22);
    invGamma_m22 = invpd(Gamma_m22);
    Kvec_b_p2 = Kvec_b_p(Kind_b_p,1);
    Kvec_b_m2 = Kvec_b_m(Kind_b_m,1);
    Psi_yyp22 = X2b_p'*diag(Kvec_b_p2.^2.*s2yypb_vec)*X2b_p/n;
    Psi_yym22 = X2b_m'*diag(Kvec_b_m2.^2.*s2yymb_vec)*X2b_m/n;
    Vp222 = 4*(e2'*invGamma_p22*Psi_yyp22*invGamma_p22*e2)/n/bcct^4;
    Vm222 = 4*(e2'*invGamma_m22*Psi_yym22*invGamma_m22*e2)/n/bcct^4;
    V222 = Vp222 + Vm222;
    C010 = n*vn*V012/4/B012^2/((e2'*betab_all_p - e2'*betab_all_m)^2+3*V222/4);
    hcct = C010^(1/5)*n^(-1/5);
    
    output.b = bcct;
    output.h = hcct;
%     [tau,delta_p,delta_m,~] = srdd(Y,X,c,hcct,hcct);
%     output.tau = tau;
%     output.delta = [delta_p delta_m];
    