%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get constants based on equivalent kernels for triangular kernel
%   See Fand and Gijbels (1996, p.64, 67)
%
% Imput:
%  v: v-th derivative of interest (strictly less than p)
%  p: order of polynomial for LPR (less than 6)
%
% Output
% Cvp: Constant based on
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Bvp] = CCT_B(v,p)

% S = (\mu_{j+k}), 0<=j,k<=p, defined in FG(96,p61), p=6
S6 = [1/2 1/6 1/12 1/20 1/30 1/42 1/56; 
       1/6 1/12 1/20 1/30 1/42 1/56 1/72;
       1/12 1/20 1/30 1/42 1/56 1/72 1/90; 
       1/20 1/30 1/42 1/56 1/72 1/90 1/110;
       1/30 1/42 1/56 1/72 1/90 1/110 1/132; 
       1/42 1/56 1/72 1/90 1/110 1/132 1/156;
       1/56 1/72 1/90 1/110 1/132 1/156 1/182];
Gamma_p = S6(1:p+1,1:p+1);

% Vector of \mu_j,
% [\mu_1, \mu_2, ..., \mu_{13}]
mu_vec = [1/6 1/12 1/20 1/30 1/42 1/56 1/72 1/90 1/110 1/132 1/156 1/182 1/210];
theta_p = mu_vec(1,p+1:2*p+1)';

Ip = eye(p+1);
ev = Ip(:,v+1);
Bvp = factorial(v)*ev'*invpd(Gamma_p)*theta_p;
   

