%--------------------------------------------------------------------
% PURPOSE: This is the main file for testing the FRD validity assumptions
% using Arai, Hsu, Kitagawa, Mourifie, and Wan (2020)
% Here we consider the MPV2013 data set.
%--------------------------------------------------------------------


diary on
tic
clc
clear



%##########################################################################
nbs=1000;           %BOOTSTRAP REPLICATIONS for calculating the critical value
rich=15;            % Parameter Q in the interval class
smooth=4.5;         % Undersmooth constant if one choose to under-smooth the MSE-optimal bandwdith.
vweight=1;                   % vweight=1 is using variance weighting. vweight=0 means no weighting.
band={'two','three','four','AI','IK','CCT'}; % bandwidth used in the main text.
% band={'AI-CER','IK-CER','CCT-CER','AI-RBC','IK-RBC','CCT-RBC'}; % other
% possible bandwdith choices, where xx-CER represents CER implementation
% based on base bandwidth choice xx, and xx-RBC represents MSE-opimtal RBC
% implementation based on base bandwidth choice xx.
if vweight==0
    XI=1;
else
    XI=[0.00999950,0.03160696,0.17058722,0.5]; % tuning parameter in the denominator of the variance estimator
end
numxi=length(XI);
nband=length(band);                    

outlist={'c_health_in_i12','c_health_out_i','var_health_in','var_health_out','c_educat_i','c_educat_f','c_food_tot','cons_total',...
    'car','radio','preventive_visit_doctor','num_growthdev_lastyear','saw_doctor_nurse_uncond','primary_care','specialist','hospitalization',...
    'see_doctor_chronic_disease','saw_doctor_nurse_uncond','activitydays','cough_fever_diarrhea','health_problem','birthwgtkg','preg_drank',...
    'preg_dranknumwk','breasted_month','preg_folic','preg_folicmths','contributivo_ECV','contributivo_DHS','insurance_special_ECV',...
    'insurance_special_DHS','uninsured_ECV','uninsured_DHS'};


numy=length(outlist);
for j=1:numy
    file=outlist{j};
    filename  = sprintf('%s.xlsx', file);
    data=xlsread(filename);
    %         filename = sprintf('%s.mat', file);
    %         load(filename);
    data(any(isnan(data),2),:)=[];
    Y=data(:,3);
    sdy=(Y-mean(Y))./std(Y);
    y=normcdf(sdy);
    x=data(:,1);
    t=data(:,2);
    c=0;
    %#########################################################################
    
    for tune=1:numxi
        xi=XI(tune);
        pval=nan(1,nband);
        Sn_vec=nan(1,nband);
        CV01=nan(1,nband);
        CV05=nan(1,nband);
        CV10=nan(1,nband);
        n=length(y);
        n_plus=sum((x>=c));
        n_minus=sum((x<c));
        B=zeros(2,length(band));
        for nb=1:nband
            bandwidth=band(nb);
            if strcmp(bandwidth,'one')==1
                bw=[1,1];
            elseif strcmp(bandwidth,'two')==1
                bw=[2,2];
            elseif strcmp(bandwidth,'three')==1
                bw=[3,3];
            elseif strcmp(bandwidth,'four')==1
                bw=[4,4];
            elseif strcmp(bandwidth,'AI')==1
                output = rddmmsef(x,y,t,c);
                bws=[output.h(2),output.h(1)];
                bw=[bws(1)*n_minus^(1/5-1/smooth),bws(2)*n_plus^(1/5-1/smooth)];
            elseif strcmp(bandwidth,'IK')==1
                bws = rd_optbandwidth_1(y,x,c);
                bw=[bws*n_minus^(1/5-1/smooth),bws*n_plus^(1/5-1/smooth)];
            elseif strcmp(bandwidth,'CCT')==1
                output = rddcct(x,y,c);
                bws=output.h;
                bw=[bws*n_minus^(1/5-1/smooth),bws*n_plus^(1/5-1/smooth)];
            elseif strcmp(bandwidth,'AI-CER')==1
                output = rddmmsef(x,y,t,c);
                bws=[output.h(2),output.h(1)];
                bw=[bws(1)*n_minus^(-1/20),bws(2)*n_plus^(-1/20)];
            elseif strcmp(bandwidth,'IK-CER')==1
                bws = rd_optbandwidth_1(y,x,c);
                bw=[bws*n_minus^(-1/20),bws*n_plus^(-1/20)];
            elseif strcmp(bandwidth,'CCT-CER')==1
                output = rddcct(x,y,c);
                bws=output.h;
                bw=[bws*n_minus^(-1/20),bws*n_plus^(-1/20)];
            elseif strcmp(bandwidth,'AI-RBC')==1
                output = rddmmsef(x,y,t,c);
                bws=[output.h(2),output.h(1)];
                bw=bws;
            elseif strcmp(bandwidth,'IK-RBC')==1
                bws = rd_optbandwidth_1(y,x,c);
                bw=[bws,bws];
            elseif strcmp(bandwidth,'CCT-RBC')==1
                output = rddcct(x,y,c);
                bws=output.h;
                bw=[bws,bws];
            end
            
            if length(bw)==2
                B(:,nb)=bw';
            else
                B(:,nb)=[bw;bw];
            end
            if strcmp(bandwidth,'AI-RBC')==1||strcmp(bandwidth,'IK-RBC')==1||strcmp(bandwidth,'CCT-RBC')==1
                [Sn_hat,P_u]=FRDTest_step_MPV_RBC(y,x,t,bw,c,nbs,vweight,xi,rich);
            else
                [Sn_hat,P_u]=FRDTest_step_MPV(y,x,t,bw,c,nbs,vweight,xi,rich);
            end
            pval(nb) = mean((P_u>=Sn_hat));
            Sn_vec(nb)=Sn_hat;
            P_u_sort=sort(P_u);
            CV01(nb)=P_u_sort(0.99*nbs);
            CV05(nb)=P_u_sort(0.95*nbs);
            CV10(nb)=P_u_sort(0.90*nbs);
        end
        
        disp('====================================================')
        disp(['outcome variable: ',outlist(j)])
        if vweight==0
            disp('no variance weighting')
        else
            disp(['variance weighting and xi=',num2str(xi)])
        end
        disp(['    bandwidth=',band])
        disp(B)
        disp(['    p-values        ',num2str(pval)])
        disp(['    Stat.         ',num2str(Sn_vec)])
        disp(['    1% CV.      ',num2str(CV01)])
        disp(['    5% CV.      ',num2str(CV05)])
        disp(['    10% CV.      ',num2str(CV10)])
    end
    
end

toc
diary off
beep;
