function [Sn_hat,P_u]= FRDTest_step_MPV_RBC(y,x,t,bw,c,nbs,vweight,xi,rich)
%--------------------------------------------------------------------
% PURPOSE: Computes the test statistic for hypothesis testing in
%          Arai, Hsu, Kitagawa, Mourifie, and Wan (2020), as well as the influence functions
%          needed for the multiplier bootstrap
%--------------------------------------------------------------------
% INPUTS:   y : outcome variable
%           x : running variable
%          bw : bandwidth
%           c : cutoff for treatment
%      nwgrid : partition of outcome range (for approx. of
%               sigma-algebra)
%--------------------------------------------------------------------
% OUTPUT:
%      Sn_hat: test statistics
%      P_u: a vector of nbs by 1 random bootstrap draws from the limiting
%           distribution
%
% NOTES: Only a triangular kernel is used
%
%--------------------------------------------------------------------

bw1=bw(1);
bw2=bw(2);
n=length(y);
n_plus=sum((x>=c));
n_minus=sum((x<c));
sn_num=(bw(1)*n_minus)*(bw(2)*n_plus);
sn_denom=(bw(1)*n_minus)+(bw(2)*n_plus);
sn=sqrt(sn_num/sn_denom);

% bw1=bw(1);
% bw2=bw(2);
% idx=(x-c)<bw2&(x-c)>-bw1;
% y=y(idx);
% x=x(idx);
% t=t(idx);
% n=length(y);
% n_plus=sum((x>=c));
% n_minus=sum((x<c));
% sn_num=(bw(1)*n_minus)*(bw(2)*n_plus);
% sn_denom=(bw(1)*n_minus)+(bw(2)*n_plus);
% sn=sqrt(sn_num/sn_denom);

an=(0.3*log(n))^(0.5);
bn=(0.4*log(n)/log(log(n)))^(0.5);

wtm=newweightm(x,c,bw1);               %Wni,- (vector from i=1 to n)
wtp=newweightp(x,c,bw2);               %Wni,+ (vector from i=1 to n)

% bwx=2.34*sqrt(var(x))/(n)^(1/5);  % RoT bandwidth for estiamting f(x) at x=c.
% fhat = sum(normpdf((x-c)./bwx))/n/bwx;




idx1= t==1&(x-c)<bw2&(x-c)>-bw1;
yob1=y(idx1);
idx0= t==0&(x-c)<bw2&(x-c)>-bw1;
yob0=y(idx0);

if length(unique(y))<15
    rich='all';
    %     disp('we treat Y as continuous variable')
else
    %     disp('we treat Y as discrete variable')
end

if strcmp(rich,'all')==1 ||strcmp(rich,'All')==1
    
    ysort1=unique(yob1);
    % L1=length(ysort1);  % L1: number of unique y-value for which D=1
    ysort0=unique(yob0);
    % L0=length(ysort0); % L1: number of unique y-value for with D=0
    
    G1=ysort1;
    N1=length(G1);
    ncubetot1=N1*(N1+1)/2; % all possible intervals associated with D=1
    
    
    G0=ysort0;
    N0=length(G0);
    ncubetot0=N0*(N0+1)/2; % all possible intervals associated with D=0
    
    nu_hat1=zeros(ncubetot1,1);             %Store estimates of nu_1
    nu_hat0=zeros(ncubetot0,1);             %Store estimates of nu_0
    % size(nu_hat1)
    nu_sd_hat1=zeros(ncubetot1,1);           % Store the standardize nu_hat1
    nu_sd_hat0=zeros(ncubetot0,1);           % Store the standardize nu_hat0
    sigma_hat1=zeros(ncubetot1,1);             %Store estimates of nu_1
    sigma_hat0=zeros(ncubetot0,1);             %Store estimates of nu_0
    %     influ1=zeros(ncubetot1,length(y));      %Store influence func. estimates
    %     influ0=zeros(ncubetot0,length(y));      %Store influence func. estimates
    %     disp([L0,N0,ncubetot0;L1,N1,ncubetot1])
    %     disp(G1)
    
    P_u=-100000000*ones(nbs,1);
    u=randn(length(y),nbs);
    G1=G1';
    G0=G0';
    
    temp=0;
    for k=1:N1
        for j=k:N1
            ygridtemp=[G1(k),G1(j)];  % create an interval A.
            catytemp=(y>=ygridtemp(1,1)).*(y<=ygridtemp(1,2));  % check each y_i and see if belongs to A.
            temp=temp+1;
            
            mhatm1=sum(wtm.*(catytemp==1).*t);  % estimate of m_{1,m}(A), for interval A.
            mhatp1=sum(wtp.*(catytemp==1).*t);  % estimate of m_{1,p}(A)
            nu_hat1(temp,1)=-mhatm1+mhatp1;  % estimate of nu_1(A),note for MPA application, the sign is different from others.
            
            phi1=-sn*(wtp.*((catytemp==1).*t-mhatp1)-wtm.*((catytemp==1).*t-mhatm1)); % calculate the vector of phi_{v1,i}(A)
            
            if vweight==1
                sigma_hat1(temp)= sum((phi1).^2);
                sd1=max(xi,sqrt(sigma_hat1(temp)));
            else
                sigma_hat1(temp)=1;
                sd1=1;
            end
            nu_sd_hat1(temp)=nu_hat1(temp,1)/sd1;
            
            psi_hat1=-bn*((sn.*nu_hat1(temp,1))<-an*sqrt(sigma_hat1(temp))); % calculate the GMS parameter for interval A
            
            
            
            P_u= max(P_u,u'*phi1./sd1+psi_hat1);
            %influ0(temp,:)=sn*(wtC.*((catytemp==1).*t-mhatC0)-wtT.*((catytemp==1).*t-mhatT0));
        end
    end
    
    temp=0;
    for k=1:N0
        for j=k:N0
            ygridtemp=[G0(k),G0(j)];  % create an interval A.
            catytemp=(y>=ygridtemp(1,1)).*(y<=ygridtemp(1,2));  % check each y_i and see if belongs to A.
            temp=temp+1;
            
            
            mhatm0=sum(wtm.*(catytemp==1).*(1-t));
            mhatp0=sum(wtp.*(catytemp==1).*(1-t));
            nu_hat0(temp,1)=-mhatp0+mhatm0;% estimate of nu_0(A),note for MPA application, the sign is different from others
            phi0=-sn*(wtm.*((catytemp==1).*(1-t)-mhatm0)-wtp.*((catytemp==1).*(1-t)-mhatp0));
            
            if vweight==1
                sigma_hat0(temp)= sum((phi0).^2);
                sd0=max(xi,sqrt(sigma_hat0(temp)));
            else
                sigma_hat0(temp)=1;
                sd0=1;
            end
            nu_sd_hat0(temp)=nu_hat0(temp,1)/sd0;
            
            
            psi_hat0=-bn*((sn.*nu_hat0(temp,1))<-an*sqrt(sigma_hat0(temp)));
            
            
            P_u= max(P_u,u'*phi0./sd0+psi_hat0);
        end
    end
    
else
    idx=find((x-c>=-bw1).*(x-c<=bw2));  % index of observations within bandwidth
    y=y(idx);
    t=t(idx);
    wtm=wtm(idx);
    wtp=wtp(idx);
    clear idx
    ncubetot=rich*(rich+1)/2;
    nu_hat1=zeros(ncubetot,1);             %Store estimates of nu_1
    nu_hat0=zeros(ncubetot,1);             %Store estimates of nu_0
    % size(nu_hat1)
    nu_sd_hat1=zeros(ncubetot,1);           % Store the standardize nu_hat1
    nu_sd_hat0=zeros(ncubetot,1);           % Store the standardize nu_hat0
    sigma_hat1=zeros(ncubetot,1);             %Store estimates of nu_1
    sigma_hat0=zeros(ncubetot,1);             %Store estimates of nu_0
    
    
    P_u=-100000000*ones(nbs,1);
    
    u=randn(length(y),nbs); % row: sample size; column: number of bootstrapped samples
    
    temp=0;
    for r=1:rich
        ygrids=min(y):((max(y)-min(y))/(r)):max(y);
        ygrids=ygrids';
        catytemp=zeros(size(y,1),1);
        for i=1:(size(ygrids,1)-1)
            catytemp=catytemp+ i.*(y>=ygrids(i,1)).*(y<=ygrids(i+1,1));
        end
        
        %         if r==rich
        %             pp1=zeros(r,1);
        %             pp0=zeros(r,1);
        
        for j=1:r
            temp=temp+1;
            mhatm1=sum(wtm.*(catytemp==j).*t);  % estimate of m_{1,m}(A), for interval A.
            mhatp1=sum(wtp.*(catytemp==j).*t);  % estimate of m_{1,p}(A)
            nu_hat1(temp,1)=-mhatm1+mhatp1;  % estimate of nu_1(A),note for MPA application, the sign is different from others.
            
            phi1=-sn*(wtp.*((catytemp==j).*t-mhatp1)-wtm.*((catytemp==j).*t-mhatm1)); % calculate the vector of phi_{v1,i}(A)
            
            if vweight==1
                sigma_hat1(temp)= sum((phi1).^2);
                sd1=max(xi,sqrt(sum((phi1).^2)));
            else
                sigma_hat1(temp)=1;
                sd1=1;
            end
            nu_sd_hat1(temp)=nu_hat1(temp,1)/sd1;
            %pp1(j)=nu_sd_hat1(temp);
            psi_hat1=-bn*((sn.*nu_hat1(temp,1))<-an*sqrt(sigma_hat1(temp))); % calculate the GMS, to move std to RHS to avoid divid zero
            
            
            
            P_u= max(P_u,(u'*phi1)/sd1+psi_hat1);
            
            
            mhatm0=sum(wtm.*(catytemp==j).*(1-t));
            mhatp0=sum(wtp.*(catytemp==j).*(1-t));
            nu_hat0(temp,1)=-mhatp0+mhatm0; % estimate of nu_0(A),note for MPA application, the sign is different from others
            phi0=-sn*(wtm.*((catytemp==j).*(1-t)-mhatm0)-wtp.*((catytemp==j).*(1-t)-mhatp0));
            
            if vweight==1
                sigma_hat0(temp)= sum((phi0).^2);
                sd0=max(xi,sqrt(sum((phi0).^2)));
            else
                sigma_hat0(temp)=1;
                sd0=1;
            end
            nu_sd_hat0(temp)=nu_hat0(temp,1)/sd0;
            
            psi_hat0=-bn*((sn.*nu_hat0(temp,1))<-an*sqrt(sigma_hat0(temp)));
            %pp0(j)=nu_sd_hat0(temp);
            
            P_u= max(P_u,(u'*phi0)/sd0+psi_hat0);
            
        end
        %plot(ygrids(1:r),pp1,ygrids(1:r),pp0,'--');
        %end
    end
    
end

Sn_hat=sn*max(max(nu_sd_hat1),max(nu_sd_hat0));  % calculate the test statistics.

return