%--------------------------------------------------------------------
% PURPOSE: This is the main file for testing the FRD validity assumptions 
% using Arai, Hsu, Kitagawa, Mourifie, and Wan (2020)
% Here we draws figures to show where the FRD assumptions are violated
% for the MPV2013 application. 
%--------------------------------------------------------------------

outlist={'c_educat_f','c_food_tot','cons_total'};
file=outlist{1}; % choose the order of the outcome variable that you want to visualize.
% filename  = sprintf('%s.mat', file);
% load(filename)
bu=2; % choose the right bandwidth.
bl=bu; % choose the left bandwidth.
filename  = sprintf('%s.xlsx', file);
data=xlsread(filename);

% I=find(data(:,3)>0);
% data=data(I,:);
% clear I;

Y=data(:,3);
sdy=(Y-mean(Y))./std(Y);
y=normcdf(sdy);
I=find(data(:,1)<bu&data(:,1)>-bl);
x=data(I,1);
t=data(I,2);
y=y(I);
dis=data(:,4);
c=0;
hb=0.015;

if length(unique(y))>15
    
    I1p=find(t==1&x>=c&x<c+bu);
    I1m=find(t==1&x<c&x>c-bl);
    I0p=find(t==0&x>=c&x<c+bu);
    I0m=find(t==0&x<c&x>c-bl);
    G=0:0.01:1;
    
    
    p1p=length(I1p)/(length(I1p)+length(I0p));
    p1m=length(I1m)/(length(I1m)+length(I0m));
    p0p=length(I0p)/(length(I1p)+length(I0p));
    p0m=length(I0m)/(length(I1m)+length(I0m));
    
    
    if strcmp(file,'c_educat_f')==1
        hb=0.015;
        f1p=ksdensity(y(I1p),G,'Bandwidth',hb,'Kernel','epanechnikov');
        f0p=ksdensity(y(I0p),G,'Bandwidth',hb,'Kernel','epanechnikov');
        f1m=ksdensity(y(I1m),G,'Bandwidth',hb,'Kernel','epanechnikov');
        f0m=ksdensity(y(I0m),G,'Bandwidth',hb,'Kernel','epanechnikov');
        figure(2)
        subplot(3,2,1)
        plot(G,f1m*p1m,'--r',G,f1p*p1p,'b'),...
            legend({'$\hat f(y,D=1|R=r^-)$','$\hat f(y,D=1|R=r^+)$'},'Interpreter','latex'),...
            grid off,xlim([0.1,1]),ylim([0,4])
        subplot(3,2,2)
        plot(G,f0m*p0m,'b',G,f0p*p0p,'--r',[0.5604,0.5604],[0,4],'-.k',[0.6693,0.6693],[0,4],':k');...
            legend({'$\hat f(y,D=0|R=r^-)$','$\hat f(y,D=0|R=r^+)$','$0.5604$','$0.6693$'},'Interpreter','latex'),...
            grid off,xlim([0.1,1]),ylim([0,4])
        subplot(3,2,3),hist(y(I1p),40),xlim([0 1]),legend({'$D=1,R=r^+$'},'Interpreter','latex'),xlim([0.1,1]),ylim([0,650])
        subplot(3,2,5),hist(y(I1m),40),xlim([0 1]),legend({'$D=1,R=r^-$'},'Interpreter','latex'),xlim([0.1,1]),ylim([0,650])
        subplot(3,2,4),hist(y(I0p),40),xlim([0 1]),legend({'$D=0,R=r^+$'},'Interpreter','latex'),xlim([0.1,1]),ylim([0,650])
        subplot(3,2,6),hist(y(I0m),40),xlim([0 1]),legend({'$D=0,R=r^-$'},'Interpreter','latex'),xlim([0.1,1]),ylim([0,650])
        I=find(y<=0.6693&y>0.5604&t==0&x<bu&x>=0);
        I_plus=length(I);
        I=find(y<=0.6693&y>0.5604&t==0&x<0&x>-bl);
        I_minus=length(I);
        disp(['# with d=0, y= [0.5604,0.6693],r>0,',num2str(I_plus),'ratio is ',num2str(I_plus/length(I0p))])
        disp(['# with d=0, y= [0.5604,0.6693],r<0,',num2str(I_minus),'ratio is ',num2str(I_minus/length(I0m))])
    elseif strcmp(file,'c_food_tot')==1
        hb=0.015;
        f1p=ksdensity(y(I1p),G,'Bandwidth',hb,'Kernel','epanechnikov');
        f0p=ksdensity(y(I0p),G,'Bandwidth',hb,'Kernel','epanechnikov');
        f1m=ksdensity(y(I1m),G,'Bandwidth',hb,'Kernel','epanechnikov');
        f0m=ksdensity(y(I0m),G,'Bandwidth',hb,'Kernel','epanechnikov');
        
        figure(2)
        subplot(3,2,1)
        plot(G,f1m*p1m,'--r',G,f1p*p1p,'b'),...
            legend({'$\hat f(y,D=1|R=r^-)$','$\hat f(y,D=1|R=r^+)$'},'Interpreter','latex'),...
            grid off,xlim([0.1,1]),ylim([0,3])
        subplot(3,2,2)
        plot(G,f0m*p0m,'b',G,f0p*p0p,'--r',[0.5948,0.5948],[0,4],'-.k',[0.6527,0.6527],[0,4],':k');...
            legend({'$\hat f(y,D=0|R=r^-)$','$\hat f(y,D=0|R=r^+)$','$0.5948$','$0.6527$'},'Interpreter','latex'),...
            grid off,xlim([0.1,1]),ylim([0,3])
        subplot(3,2,3),hist(y(I1p),40),xlim([0 1]),legend({'$D=1,R=r^+$'},'Interpreter','latex'),xlim([0.1,1]),ylim([0,130])
        subplot(3,2,5),hist(y(I1m),40),xlim([0 1]),legend({'$D=1,R=r^-$'},'Interpreter','latex'),xlim([0.1,1]),ylim([0,130])
        subplot(3,2,4),hist(y(I0p),40),xlim([0 1]),legend({'$D=0,R=r^+$'},'Interpreter','latex'),xlim([0.1,1]),ylim([0,130])
        subplot(3,2,6),hist(y(I0m),40),xlim([0 1]),legend({'$D=0,R=r^-$'},'Interpreter','latex'),xlim([0.1,1]),ylim([0,130])
        I=find(y<=0.6527&y>0.5948&t==0);
        I_total=length(I);
        I=find(y<=0.6527&y>0.5948&t==0&x<bu&x>=0);
        I_plus=length(I);
        I=find(y<=0.6527&y>0.5948&t==0&x<0&x>-bl);
        I_minus=length(I);
        disp(['# with d=0, y= [0.5948,0.6527],r>0,',num2str(I_plus),'ratio is ',num2str(I_plus/length(I0p))])
        disp(['# with d=0, y= [0.5948,0.6527],r<0,',num2str(I_minus),'ratio is ',num2str(I_minus/length(I0m))])
    else
        hb=0.02;
        f1p=ksdensity(y(I1p),G,'Bandwidth',hb,'Kernel','epanechnikov');
        f0p=ksdensity(y(I0p),G,'Bandwidth',hb,'Kernel','epanechnikov');
        f1m=ksdensity(y(I1m),G,'Bandwidth',hb,'Kernel','epanechnikov');
        f0m=ksdensity(y(I0m),G,'Bandwidth',hb,'Kernel','epanechnikov');
        figure(2)
        title(file)
        subplot(3,2,1)
        plot(G,f1m*p1m,'--r',G,f1p*p1p,'b',[0.3042,0.3042],[0,5],'-.k',[0.3737,0.3737],[0,5],':k'),...
            legend({'$\hat f(y,D=1|R=r^-)$','$\hat f(y,D=1|R=r^+)$','$0.3042$','$0.3737$'},'Interpreter','latex'),...
            grid off,xlim([0.1,1]),ylim([0,5])
        subplot(3,2,2)
        plot(G,f0m*p0m,'b',G,f0p*p0p,'--r');...
            legend({'$\hat f(y,D=0|R=r^-)$','$\hat f(y,D=0|R=r^+)$'},'Interpreter','latex'),...
            grid off,xlim([0.1,1]),ylim([0,5])
        subplot(3,2,3),hist(y(I1p),40),xlim([0 1]),legend({'$D=1,R=r^+$'},'Interpreter','latex'),xlim([0.1,1]),ylim([0,60])
        subplot(3,2,5),hist(y(I1m),40),xlim([0 1]),legend({'$D=1,R=r^-$'},'Interpreter','latex'),xlim([0.1,1]),ylim([0,60])
        subplot(3,2,4),hist(y(I0p),40),xlim([0 1]),legend({'$D=0,R=r^+$'},'Interpreter','latex'),xlim([0.1,1]),ylim([0,60])
        subplot(3,2,6),hist(y(I0m),40),xlim([0 1]),legend({'$D=0,R=r^-$'},'Interpreter','latex'),xlim([0.1,1]),ylim([0,60])
        I=find(y<=0.3737&y>0.3042&t==1);
        I_total=length(I);
        I=find(y<=0.3737&y>0.3042&t==1&x<bu&x>=0);
        I_plus=length(I);
        I=find(y<=0.3737&y>0.3042&t==1&x<0&x>-bl);
        I_minus=length(I);
        disp(['# with d=1, y= [0.3042,0.3737],r>0,',num2str(I_plus),'ratio is ',num2str(I_plus/length(I1p))])
        disp(['# with d=1, y= [0.3042,0.3737],r<0,',num2str(I_minus),'ratio is ',num2str(I_minus/length(I1m))])
    end
    
else
    uy=unique(y);
    N=length(uy);
    f1p=zeros(N,1);
    f0p=f1p;
    f0m=f1p;
    f1m=f1p;
    for i=1:N
        f1p(i)=sum((t==1).*(y==uy(i)).*(x>=c).*(x<c+bu))/sum((x>=c).*(x<c+bu));
        f0p(i)=sum((t==0).*(y==uy(i)).*(x>=c).*(x<c+bu))/sum((x>=c).*(x<c+bu));
        f1m(i)=sum((t==1).*(y==uy(i)).*(x>c-bl).*(x<c))/sum((x>c-bl).*(x<c));
        f0m(i)=sum((t==0).*(y==uy(i)).*(x>c-bl).*(x<c))/sum((x>c-bl).*(x<c));
    end
    
    G=uy;
    
    
    
    subplot(3,2,1),plot(G,f1m,'--k',G,f1p,'k');legend({'$P(Y=y,D=1|R=r^-)$','$P(Y=y,D=1|R=r^+)$'},'Interpreter','latex')
    
    subplot(3,2,2),plot(G,f0m,'k',G,f0p,'--k');legend({'$P(Y=y,D=0|R=r^-)$','$P(Y=y,D=0|R=r^+)$'},'Interpreter','latex')
    
end
