%--------------------------------------------------------------------
% PURPOSE: This is the main simulation file for testing the FRD validity assumptions
% using Arai, Hsu, Kitagawa, Mourifie, and Wan (2018)
%--------------------------------------------------------------------
clc
clear
rng(12345)
tic
%##########################################################################
c=0;                        %THRESHOLD
nbs=300;                   %BOOTSTRAP REPLICATIONS
rich=15;                     % rich=all: all intervals. rich=q, coarse intervals based on L=q.
dgp_cat='Size';              %DGP CATEGORY: 'Size' or 'Power'.
smooth=4.5;                   % undersmoothing constant
numsim=1000;                  %MONTE CARLO REPLICATIONS
vweight=1;                   % 0: no weighting, 1 weighting
nsize=[1000,2000,4000,8000];
numn=length(nsize);
if strcmp(dgp_cat,'Size')==1
    numdgp=3;
else
    numdgp=6;
end
progress='Off';  % display progress or not, choose On or Off
Rpath='C:\Program Files\R\R-4.0.1\bin'; % specify location of R program
RscriptFileName='D:\Dropbox\Code and Data for FRD Test\simulation\dpi_bandwidth.R'; % specify location of CEDPI R file that estimates bandwidths using NPROBUST package
% if using R program for first time, please open it and change the path to the R file as instructed.
% also, add the Rpath to the Matlab path so that Matlab can call R.

%##########################################################################
count=0;
for dgp=1:numdgp
    
    rejprop1=nan(numn,1);
    rejprop5=nan(numn,1);
    rejprop10=nan(numn,1);
    avgbw=nan(numn,2);
    
    
    
    for nind=1:numn
        
        n=nsize(nind);
        bw_sim=nan(numsim,2);
        
        
        rej1_sim=nan(numsim,1);
        rej5_sim=nan(numsim,1);
        rej10_sim=nan(numsim,1);
        Sn_vec=nan(numsim,1);
        
        for s=1:numsim
            disp(['n=',num2str(n),', s=',num2str(s)])
            count=count+1;
            Percent_Complete=sprintf('%.3f%%',count/(numn*numsim*1*numdgp*1)*100);
            %                     if strcmp(progress,'On')==1
            %                         disp(Percent_Complete)
            %                     end
            [bwmat,rej1,rej5,rej10,Sn_hat]=step1_CER(dgp,dgp_cat,n,c,nbs,vweight,rich,Rpath,RscriptFileName);
            bw_sim(s,:)=bwmat;
            rej1_sim(s,:)=rej1;
            rej5_sim(s,:)=rej5;
            rej10_sim(s,:)=rej10;
            %         disp(sum(rej10_sim(1:s,:))/s);
            Sn_vec(s,:)=Sn_hat;
        end
        
        avgbw(nind,:)=mean(bw_sim);
        rejprop1(nind,:)=mean(rej1_sim);
        rejprop5(nind,:)=mean(rej5_sim);
        rejprop10(nind,:)=mean(rej10_sim);
        
    end
    disp('=============================')
    disp(['DGP=',dgp_cat,num2str(dgp),'  weight=',num2str(vweight),'  L=',num2str(rich)]);
    
    disp('CEDPI')
    disp(avgbw);
    disp('      1%        5%       10%')
    disp([rejprop1,rejprop5,rejprop10])
    
end

toc

beep;
