%--------------------------------------------------------------------
% PURPOSE: This is the main simulation file for testing the FRD validity assumptions
% using Arai, Hsu, Kitagawa, Mourifie, and Wan (2020)
%--------------------------------------------------------------------
clc
clear
rng(12345)
tic
%##########################################################################
c=0;                        %THRESHOLD
nbs=300;                   %BOOTSTRAP REPLICATIONS
rich=15;                     % rich=all: all intervals. rich=q, coarse intervals based on L=q.
smooth=4.5;                   % undersmoothing constant
numsim=1000;                  %MONTE CARLO REPLICATIONS
vweight=1;                   % 0: no weighting, 1 weighting
n=8000;
D=0:0.02:0.5;      % propensity score jump size parameter
band={'IK'};
Bandtypes={'US'};
nband=length(band);
nbandtype=length(Bandtypes);
nD=length(D);

%##########################################################################


rejprop1=nan(nD*nband,nbandtype);
rejprop5=nan(nD*nband,nbandtype);
rejprop10=nan(nD*nband,nbandtype);
Jumpsize=nan(nD*nband,1);
for nb=1:nband
    bandwidth=band(nb);
    if strcmp(bandwidth,'ROT')==1 ||strcmp(bandwidth,'AI')==1
        avgbw=nan(nD,2*nbandtype);
    else
        avgbw=nan(nD,nbandtype);
    end
    
    
    for dj=1:nD
        
        d=D(dj);% propensity score jump size is 2d. 
        
        if strcmp(bandwidth,'ROT')==1 ||strcmp(bandwidth,'AI')==1
            bw_sim=nan(numsim,2*nbandtype);
        else
            bw_sim=nan(numsim,nbandtype);
        end
        
        rej1_sim=nan(numsim,nbandtype);
        rej5_sim=nan(numsim,nbandtype);
        rej10_sim=nan(numsim,nbandtype);
        Sn_vec=nan(numsim,nbandtype);
        
        for s=1:numsim
            [bwmat,rej1,rej5,rej10,Sn_hat,js]=step1_jump(bandwidth,n,c,nbs,vweight,rich,Bandtypes,smooth,d);
            bw_sim(s,:)=bwmat;
            rej1_sim(s,:)=rej1;
            rej5_sim(s,:)=rej5;
            rej10_sim(s,:)=rej10;
            %         disp(sum(rej10_sim(1:s,:))/s);
            Sn_vec(s,:)=Sn_hat;
        end
        
        avgbw(dj,:)=mean(bw_sim);
        rejprop1(dj+(nb-1)*nD,:)=mean(rej1_sim,1);
        rejprop5(dj+(nb-1)*nD,:)=mean(rej5_sim,1);
        rejprop10(dj+(nb-1)*nD,:)=mean(rej10_sim,1);
        Jumpsize(dj+(nb-1)*nD,1)=js;
        
    end
    
    for nt=1:nbandtype
        disp([bandwidth,Bandtypes(nt)])
        Outputdata=[Jumpsize(1+(nb-1)*nD:nb*nD,1),rejprop1(1+(nb-1)*nD:nb*nD,nt),rejprop5(1+(nb-1)*nD:nb*nD,nt),rejprop10(1+(nb-1)*nD:nb*nD,nt)];
        disp('Jump Size      1%        5%       10%')
        disp(Outputdata)
    end
end

toc
beep;
