%--------------------------------------------------------------------
% PURPOSE: This is the main simulation file for testing the FRD validity assumptions
% using Arai, Hsu, Kitagawa, Mourifie, and Wan (2018)
%--------------------------------------------------------------------

clc
clear
rng(12345)
tic
%##########################################################################
c=0;                        %THRESHOLD
nbs=300;                   %BOOTSTRAP REPLICATIONS
rich=15;                     % rich=all: all intervals. rich=Q, coarse intervals based on L=q.
dgp_cat='Size';              %DGP CATEGORY: 'Size' or 'Power'. Size contains DGPs that satisfy FRD assumptions. Power contains DGPs that violates FRD assumptions.
smooth=4.5;                   % undersmoothing constant
numsim=1000;                  %MONTE CARLO REPLICATIONS
vweight=1;                   % 0: no weighting, 1 weighting in the variance of the test statistics. 
nsize=[1000,2000,4000,8000]; % four sample sizes
band={'AI','IK','CCT'};   % selecting bandwidth. 
Bandtypes={'US','CER','RBC'};   % selecting further tuning on bandwidth. US: undersmoothing, CER: coverage error optimal, RBC: MSE-optimal Robust bias correction
nband=length(band);
nbandtype=length(Bandtypes);
numn=length(nsize);
if strcmp(dgp_cat,'Size')==1
    numdgp=3;
else
    numdgp=6;
end

%##########################################################################
count=0;
for dgp=1:numdgp
    rejprop1=nan(numn*nband,nbandtype);
    rejprop5=nan(numn*nband,nbandtype);
    rejprop10=nan(numn*nband,nbandtype);
    for nb=1:nband
        bandwidth=band(nb);
        if strcmp(bandwidth,'ROT')==1 ||strcmp(bandwidth,'AI')==1
            avgbw=nan(numn,2*nbandtype);
        else
            avgbw=nan(numn,nbandtype);
        end
        
        
        for nind=1:numn
            
            n=nsize(nind);
            
            if strcmp(bandwidth,'ROT')==1 ||strcmp(bandwidth,'AI')==1
                bw_sim=nan(numsim,2*nbandtype);
            else
                bw_sim=nan(numsim,nbandtype);
            end
            
            rej1_sim=nan(numsim,nbandtype);
            rej5_sim=nan(numsim,nbandtype);
            rej10_sim=nan(numsim,nbandtype);
            Sn_vec=nan(numsim,nbandtype);
            
            for s=1:numsim
                count=count+1;
                Percent_Complete=sprintf('%.3f%%',count/(numn*numsim*nband*numdgp*nbandtype)*100);
                [bwmat,rej1,rej5,rej10,Sn_hat]=step1(dgp,dgp_cat,bandwidth,n,c,nbs,vweight,rich,Bandtypes,smooth);
                bw_sim(s,:)=bwmat;
                rej1_sim(s,:)=rej1;
                rej5_sim(s,:)=rej5;
                rej10_sim(s,:)=rej10;
                Sn_vec(s,:)=Sn_hat;
            end
            
            avgbw(nind,:)=mean(bw_sim);
            rejprop1(nind+(nb-1)*numn,:)=mean(rej1_sim);
            rejprop5(nind+(nb-1)*numn,:)=mean(rej5_sim);
            rejprop10(nind+(nb-1)*numn,:)=mean(rej10_sim);
            
        end
        disp('=============================')
        disp(['DGP=',dgp_cat,num2str(dgp),'  weight=',num2str(vweight),'  L=',num2str(rich)]);
        for nt=1:nbandtype
            disp([bandwidth,Bandtypes(nt)])
            if strcmp(bandwidth,'ROT')==1 ||strcmp(bandwidth,'AI')==1
                disp(avgbw(:,(nt-1)*2+1:(nt-1)*2+2));
            else
                disp(avgbw(:,nt));
            end
            disp('      1%        5%       10%')
            disp([rejprop1(1+(nb-1)*numn:nb*numn,nt),rejprop5(1+(nb-1)*numn:nb*numn,nt),rejprop10(1+(nb-1)*numn:nb*numn,nt)])
        end
    end
end
toc

beep;
