function [bwmat,rej1,rej5,rej10,Sn_hat]=step1(dgp,dgp_cat,...
    bandwidth,n,c,nbs,vweight,rich,Bandtypes,smooth)


[y,t,x]=dgpselect(dgp,dgp_cat,c,n);
n_plus=sum((x>=c));
n_minus=sum((x<c));
nbandtype=length(Bandtypes);
Sn_hat=nan(1,nbandtype);
rej1=nan(1,nbandtype);
rej5=nan(1,nbandtype);
rej10=nan(1,nbandtype);

if strcmp(bandwidth,'ROT')==1 ||strcmp(bandwidth,'AI')==1
    bwmat=nan(1,2*nbandtype);
else
    bwmat=nan(1,nbandtype);
end

if strcmp(bandwidth,'IK')==1 
    %IK BANDWIDTH SELECTION PROCEDURE FROM IMBENS AND KALYANARAMAN (2012)
    bws = rd_optbandwidth_1(y,x,c);
    % %     truncate sample to speed simulation
    y1=y;
    t1=t;
    x1=x;

    for j=1:nbandtype
        %OUTER LOOP ITERATES OVER bandwidth type (US, CER, RBC)
        bandtype=Bandtypes(j);    
        if strcmp(bandtype,'US')==1
            bw=bws*n^(1/5-1/smooth);
            [Sn_hat(j),P_u]=step2(y1,x1,t1,bw,c,nbs,vweight,rich);
        elseif strcmp(bandtype,'CER')==1
            bw=bws*n^(-1/20);
            [Sn_hat(j),P_u]=step2(y1,x1,t1,bw,c,nbs,vweight,rich);
        elseif strcmp(bandtype,'RBC')==1
            bw=bws;
            [Sn_hat(j),P_u]=step2_rbc(y1,x1,t1,bw,c,nbs,vweight,rich);
        end
        %TEST RDD ASSUMPTIONS
        
        eta=10^(-6);
        %RECORD RESULTS OF TEST
        bwmat(j)=bw;
        rej1(j)=(Sn_hat(j)>(quantile(P_u,0.99+eta)+eta));
        rej5(j)=(Sn_hat(j)>(quantile(P_u,0.95+eta)+eta));
        rej10(j)=(Sn_hat(j)>(quantile(P_u,0.90+eta)+eta));
    end
    

elseif strcmp(bandwidth,'AI')==1
    %AI BANDWIDTH SELECTION PROCEDURE
    output = rddmmsef(x,y,t,c);
    bws=[output.h(2),output.h(1)]; %h(2)=left of cutoff, h(1)=right of cutoff
    % %     truncate sample to speed simulation
    y1=y;
    t1=t;
    x1=x;
    for j=1:nbandtype
        %OUTER LOOP ITERATES OVER bandwidth type (US, CER, RBC)
        bandtype=Bandtypes(j);
        if strcmp(bandtype,'US')==1
            bw=[bws(1)*n_minus^(1/5-1/smooth),bws(2)*n_plus^(1/5-1/smooth)];
            [Sn_hat(j),P_u]=step2(y1,x1,t1,bw,c,nbs,vweight,rich);
        elseif strcmp(bandtype,'CER')==1
            [Sn_hat(j),P_u]=step2(y1,x1,t1,bw,c,nbs,vweight,rich);
            bw=[bws(1)*n_minus^(-1/20),bws(2)*n_plus^(-1/20)];
        elseif strcmp(bandtype,'RBC')==1
            bw=bws;
            [Sn_hat(j),P_u]=step2_rbc(y1,x1,t1,bw,c,nbs,vweight,rich);
        end

        %TEST RDD ASSUMPTIONS
        
        eta=10^(-6);
        
        %RECORD RESULTS OF TEST
        bwmat(1,1+(j-1)*2:2+(j-1)*2)=bw;
        rej1(j)=(Sn_hat(j)>(quantile(P_u,0.99+eta)+eta));
        rej5(j)=(Sn_hat(j)>(quantile(P_u,0.95+eta)+eta));
        rej10(j)=(Sn_hat(j)>(quantile(P_u,0.90+eta)+eta));
    end
    
elseif strcmp(bandwidth,'CCT')==1
    %CCT BANDWIDTH SELECTION PROCEDURE FROM IMBENS AND KALYANARAMAN (2012)
    output = rddcct(x,y,c);
    bws=output.h;
    % %     truncate sample to speed simulation
    y1=y;
    t1=t;
    x1=x;
    for j=1:nbandtype
        %OUTER LOOP ITERATES OVER bandwidth type (US, CER, RBC)
        bandtype=Bandtypes(j);
        if strcmp(bandtype,'US')==1
            bw=bws*n^(1/5-1/smooth);
            [Sn_hat(j),P_u]=step2(y1,x1,t1,bw,c,nbs,vweight,rich);
        elseif strcmp(bandtype,'CER')==1
            bw=bws*n_minus^(-1/20);
            [Sn_hat(j),P_u]=step2(y1,x1,t1,bw,c,nbs,vweight,rich);
        elseif strcmp(bandtype,'RBC')==1
            bw=bws;
            [Sn_hat(j),P_u]=step2_rbc(y1,x1,t1,bw,c,nbs,vweight,rich);
        end
        
        %TEST RDD ASSUMPTIONS
        
        eta=10^(-6);
        
        %RECORD RESULTS OF TEST
        bwmat(j)=bw;
        rej1(j)=(Sn_hat(j)>(quantile(P_u,0.99+eta)+eta));
        rej5(j)=(Sn_hat(j)>(quantile(P_u,0.95+eta)+eta));
        rej10(j)=(Sn_hat(j)>(quantile(P_u,0.90+eta)+eta));  
    end
    
    
elseif strcmp(bandwidth,'ROT')==1
    
    
    bws=ROTbw(y,x,c);
    % %     truncate sample to speed simulation
    y1=y;
    t1=t;
    x1=x;
    for j=1:nbandtype
        bandtype=Bandtypes(j);
        
        if strcmp(bandtype,'US')==1
            %OUTER LOOP ITERATES OVER UNDERSMOOTHING CONSTANTS C=30/11, 4.5, 5
            bw=[bws(1)*n_minus^(1/5-1/smooth),bws(2)*n_plus^(1/5-1/smooth)];
            [Sn_hat(j),P_u]=step2(y1,x1,t1,bw,c,nbs,vweight,rich);
        elseif strcmp(bandtype,'CER')==1
            bw=[bws(1)*n_minus^(-1/20),bws(2)*n_plus^(-1/20)];
            [Sn_hat(j),P_u]=step2(y1,x1,t1,bw,c,nbs,vweight,rich);
        elseif strcmp(bandtype,'RBC')==1
            bw=bws;
            [Sn_hat(j),P_u]=step2_rbc(y1,x1,t1,bw,c,nbs,vweight,rich);
        end
        
        
        %TEST RDD ASSUMPTIONS
        eta=10^(-6);
        
        %RECORD RESULTS OF TEST
        bwmat(1,1+(j-1)*2:2+(j-1)*2)=bw;
        rej1(j)=(Sn_hat(j)>(quantile(P_u,0.99+eta)+eta));
        rej5(j)=(Sn_hat(j)>(quantile(P_u,0.95+eta)+eta));
        rej10(j)=(Sn_hat(j)>(quantile(P_u,0.90+eta)+eta));
        
    end
    
end


return